/*
DictionaryForMIDs - a free multi-language dictionary for mobile devices.
Copyright (C) 2005 Gert Nuber (Gert.Nuber@gmx.net)

GPL applies - see file COPYING for copyright statement.
*/

package de.kugihan.dictionaryformids.dataaccess;

import de.kugihan.dictionaryformids.general.*;
import de.kugihan.dictionaryformids.translation.Normation;

public class DictionaryDataFile  {

	/*
	 * describes the structure of the dictionary data files
	 */ 
	
	public static int numberOfAvailableLanguages;	
	public static int numberOfInputLanguages;
	
	public static LanguageDefinition [] supportedLanguages;
	
	public static String infoText;
	public static String dictionaryAbbreviation;

	public static String pathNameDataFiles = "dictionary";
	public static String prefixSearchListFile = "searchlist";
	public static String suffixSearchListFile = ".csv";
	public static String searchListCharEncoding;
	public static char   searchListFileSeparationCharacter;
	public static int    searchListFileMaxSize;
	public static String prefixIndexFile = "index";
	public static String suffixIndexFile = ".csv";
	public static String indexCharEncoding;
	public static char   indexFileSeparationCharacter;
	public static char   indexFileSeparatorFileNumberToPosition = '-';
	public static char   indexFileSeparatorFilePositionToSearchIndicator = '-';
	public static char   indexFileSeparatorIndexEntries = ',';
	public static int    indexFileMaxSize;
	public static String prefixDictionaryFile = "directory";
	public static String suffixDictionaryFile = ".csv";
	public static String dictionaryCharEncoding;
	public static char   dictionaryFileSeparationCharacter;
	public static int    dictionaryFileMaxSize;
	public static String applicationFileNamePrefix = "DictionaryForMIDs";
	public static String propertyFileName = applicationFileNamePrefix + ".properties";

	// properties used by DictionaryGeneration only
	public static String dictionaryGenerationInputCharEncoding;
	public static char   dictionaryGenerationSeparatorCharacter;
	public static long   dictionaryGenerationMinNumberOfEntriesPerDictionaryFile;
	public static long   dictionaryGenerationMinNumberOfEntriesPerIndexFile;
	
	public static String fileEncodingFormat; // for "special use" - currently ignored
	

	public static void initValues(boolean initDictionaryGenerationValues)
	throws Throwable
	{
		/* 
		 * values from resource file 
		 * */
		Util utilObj = Util.getUtil();
		utilObj.openProperties();
		infoText = utilObj.getDictionaryPropertyString("infoText");
		dictionaryAbbreviation = utilObj.getDictionaryPropertyString("dictionaryAbbreviation", true);
		checkForEmptyProperty(dictionaryAbbreviation);
		numberOfAvailableLanguages = utilObj.getDictionaryPropertyInt("numberOfAvailableLanguages");
		numberOfInputLanguages = 0; // initialised with 0
		supportedLanguages = new LanguageDefinition[numberOfAvailableLanguages];
		for (int indexLanguage = 0;
	         indexLanguage < numberOfAvailableLanguages;
	         ++indexLanguage) {
			String indexLanguageString = String.valueOf(indexLanguage + 1);
			String languagePropertyPrefix = "language" + indexLanguageString;
			String languageDisplayText = utilObj.getDictionaryPropertyString(languagePropertyPrefix + "DisplayText");
			String languageFilePostfix = utilObj.getDictionaryPropertyString(languagePropertyPrefix + "FilePostfix");
			boolean isSearchable = utilObj.getDictionaryPropertyBooleanDefault(languagePropertyPrefix + "IsSearchable", true);
			if (isSearchable)
				++numberOfInputLanguages;
			boolean hasSeparateDictionaryFile = utilObj.getDictionaryPropertyBooleanDefault(languagePropertyPrefix + "HasSeparateDictionaryFile", false);
			boolean generateIndex = utilObj.getDictionaryPropertyBooleanDefault(languagePropertyPrefix + "GenerateIndex", true);
			String normationClassName = utilObj.getDictionaryPropertyString(languagePropertyPrefix + "NormationClassName", true);
			checkForEmptyProperty(normationClassName);
			String dictionaryUpdateClassName = utilObj.getDictionaryPropertyString(languagePropertyPrefix + "DictionaryUpdateClassName", true);
			checkForEmptyProperty(dictionaryUpdateClassName);
			String expressionSplitString = null;
			if (initDictionaryGenerationValues) {
				expressionSplitString = utilObj.getDictionaryPropertyString("dictionaryGenerationLanguage" + indexLanguageString + "ExpressionSplitString", true);
				checkForEmptyProperty(expressionSplitString);
			}
			supportedLanguages[indexLanguage] = new LanguageDefinition(languageDisplayText, 
					                                                   languageFilePostfix,
																	   isSearchable,
																	   hasSeparateDictionaryFile,
					                                                   normationClassName,
					                                                   dictionaryUpdateClassName,
					                                                   generateIndex,
					                                                   expressionSplitString);
	    }
		searchListCharEncoding = utilObj.getDictionaryPropertyString("searchListCharEncoding");
		utilObj.setDeviceCharEncoding(searchListCharEncoding);
		searchListFileSeparationCharacter = utilObj.getDictionaryPropertyChar("searchListFileSeparationCharacter");
		searchListFileMaxSize = utilObj.getDictionaryPropertyIntDefault("searchListFileMaxSize", 10000);
		indexCharEncoding = utilObj.getDictionaryPropertyString("indexCharEncoding");
		utilObj.setDeviceCharEncoding(indexCharEncoding);
		indexFileSeparationCharacter = utilObj.getDictionaryPropertyChar("indexFileSeparationCharacter");
		indexFileMaxSize = utilObj.getDictionaryPropertyIntDefault("indexFileMaxSize", 10000);
		dictionaryCharEncoding = utilObj.getDictionaryPropertyString("dictionaryCharEncoding");
		utilObj.setDeviceCharEncoding(dictionaryCharEncoding);
		dictionaryFileSeparationCharacter = utilObj.getDictionaryPropertyChar("dictionaryFileSeparationCharacter");			
		dictionaryFileMaxSize = utilObj.getDictionaryPropertyIntDefault("dictionaryFileMaxSize", 10000);
		if (initDictionaryGenerationValues) {
			dictionaryGenerationInputCharEncoding = utilObj.getDictionaryPropertyStringDefault("dictionaryGenerationInputCharEncoding",  "ISO-8859-1");
			dictionaryGenerationSeparatorCharacter = utilObj.getDictionaryPropertyCharDefault("dictionaryGenerationSeparatorCharacter", '\t');
			dictionaryGenerationMinNumberOfEntriesPerDictionaryFile = utilObj.getDictionaryPropertyIntDefault("dictionaryGenerationMinNumberOfEntriesPerDictionaryFile", 200);
			dictionaryGenerationMinNumberOfEntriesPerIndexFile = utilObj.getDictionaryPropertyIntDefault("dictionaryGenerationMinNumberOfEntriesPerIndexFile", 500);
		}

		fileEncodingFormat = utilObj.getDictionaryPropertyString("fileEncodingFormat", true);
		if (fileEncodingFormat == null) {
			Util.getUtil().logDebug("Property fileEncodingFormat not found");
			fileEncodingFormat = new String("plain_format1");
		}
		utilObj.closeProperties();

		/* 
		 * load normation classes and create normation objects; same for dictionaryUpdate classes/objects
		 * */
		for (int indexLanguage = 0;
	         indexLanguage < numberOfAvailableLanguages;
			 ++indexLanguage) {
			String normationClassName = supportedLanguages[indexLanguage].normationClassName;
			if (normationClassName == null) {
				// no normation class defined: use default normation class
				normationClassName = "de.kugihan.dictionaryformids.translation.Normation";
			}
			Class normationClass = Class.forName(normationClassName);
			supportedLanguages[indexLanguage].normationObj = (Normation) normationClass.newInstance(); 
			
			if (initDictionaryGenerationValues) {
				String dictionaryUpdateClassName = supportedLanguages[indexLanguage].dictionaryUpdateClassName;
				if (dictionaryUpdateClassName == null) {
					// no normation class defined: use default normation class
					dictionaryUpdateClassName = "de.kugihan.dictionaryformids.dictgen.DictionaryUpdate";
				}
				Class dictionaryUpdateClass = Class.forName(dictionaryUpdateClassName);
				supportedLanguages[indexLanguage].dictionaryUpdateObj = (DictionaryUpdateIF) dictionaryUpdateClass.newInstance();
			}
		}
	}
	
	// if property is provided but empty, then this is handled as if no property was provided:
	static void checkForEmptyProperty(String propertyName) {
		if (propertyName != null)
			if (propertyName.length() == 0)
				propertyName = null;
	}
	
	public static String getPathDataFiles()
	{
		return "/" + pathNameDataFiles + "/";
	}
}
