/*
DictionaryForMIDs - a free multi-language dictionary for mobile devices.
Copyright (C) 2005 Gert Nuber (Gert.Nuber@gmx.net)

GPL applies - see file COPYING for copyright statement.
*/

package de.kugihan.dictionaryformids.general;
import java.io.*; 
import javax.microedition.rms.*; 
import javax.microedition.lcdui.*; 

import de.kugihan.dictionaryformids.dataaccess.DictionaryDataFile;
import de.kugihan.dictionaryformids.hmi_j2me.DictionaryForMIDs;

public class SettingsStore {

	RecordStore mobileDictionaryStore;
	static SettingsStore instanceSettingsStore = null;

	private final int rms_index_structure_version = 1;
	private final int rms_index_input_language_group = rms_index_structure_version + 1;
	private final int rms_index_output_language_group = rms_index_input_language_group + 1;
	private final int rms_index_statistic = rms_index_output_language_group + 1;
	private final int rms_index_bypassCharsetDecoding = rms_index_statistic + 1;
	private final int rms_index_incrementalSearchDisabled = rms_index_bypassCharsetDecoding + 1;
	private final int rms_index_maxHits = rms_index_incrementalSearchDisabled + 1;
	private final int rms_index_fontSize = rms_index_maxHits + 1;
	private final int rms_max_index = rms_index_fontSize;

	public static SettingsStore getSettingsStore() throws DictionaryException {
		if (instanceSettingsStore == null)
			instanceSettingsStore = new SettingsStore();
		return instanceSettingsStore;
	}
	
	private final String rmsStoreName = "DictionaryForMIDs";

	public SettingsStore() throws DictionaryException {
		// open RMS store
		try {
			mobileDictionaryStore = RecordStore.openRecordStore(rmsStoreName, true);
			boolean buildNewRMSStore = false;
			if (mobileDictionaryStore.getNumRecords() < rms_max_index) {
				buildNewRMSStore = true;
			}
			else {
				byte [] structureVersionRecord = mobileDictionaryStore.getRecord(rms_index_structure_version);
				byte structureVersion = structureVersionRecord[0];
				if (structureVersion != DictionaryForMIDs.versionRMSStructure)
					buildNewRMSStore = true;					
			}
			if (buildNewRMSStore) {
				// Re-Initialize Store: delete it and create from scratch
				mobileDictionaryStore.closeRecordStore();
				RecordStore.deleteRecordStore(rmsStoreName);
				mobileDictionaryStore = RecordStore.openRecordStore(rmsStoreName, true);
				// 
				// set up intital field values
				//
				// store structure version
				mobileDictionaryStore.addRecord(new byte [] { DictionaryForMIDs.versionRMSStructure }, 0, 1);
				// inputLanguage: first available language (index 0):
				mobileDictionaryStore.addRecord(new byte [] { 0 }, 0, 1);
				// outputLanguages: the second available language selected:
				try {
					boolean [] outputLanguage = new boolean[DictionaryDataFile.numberOfAvailableLanguages];
					for (int languageCounter = 0; 
				         languageCounter < DictionaryDataFile.numberOfAvailableLanguages; 
					     ++languageCounter) {
						boolean languageSetting = languageCounter == 1;
						outputLanguage[languageCounter] = languageSetting;
					}
					byte [] outputLanguageRecord = getBooleanArrayRecord(outputLanguage);
					mobileDictionaryStore.addRecord(outputLanguageRecord,
							                        0,
							                        outputLanguageRecord.length);
				}
				catch (IOException e) {
					throw new DictionaryException(e);
				}
				// statistics settings: switched off (0 is off/false; 1 is on/true)
				mobileDictionaryStore.addRecord(new byte [] { 0 }, 0, 1);
				// BypassCharsetDecoding settings: switched off (0 is off/false; 1 is on/true)
				mobileDictionaryStore.addRecord(new byte [] { 0 }, 0, 1);
				// incremental search disabled settings: switched on (0 is off/false; 1 is on/true)
				mobileDictionaryStore.addRecord(new byte [] { 1 }, 0, 1);
				// maxHits settings: 30
				mobileDictionaryStore.addRecord(new byte [] { 30 }, 0, 1);
				// Font settings: device default settings
				mobileDictionaryStore.addRecord(new byte [] { 0 }, 0, 1);
			}
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
	}
	
	public void closeSettingsStore() throws DictionaryException {
		try {
			mobileDictionaryStore.closeRecordStore();
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
	}
	
	public int getInputLanguage() throws DictionaryException  {
		int inputLanguage;
		try {
			 byte [] inputLanguageRecord = mobileDictionaryStore.getRecord(rms_index_input_language_group);
			 inputLanguage = inputLanguageRecord[0];
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
		return inputLanguage;
	}
	
	public void setInputLanguage(int inputLanguage) throws DictionaryException  {
		try {
			mobileDictionaryStore.setRecord(rms_index_input_language_group, 
			                                new byte [] { (byte) inputLanguage },
					                        0,
					                        1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
	}
	
	public boolean [] getOutputLanguage() throws DictionaryException  {
		boolean [] outputLanguage = new boolean[DictionaryDataFile.numberOfAvailableLanguages];
		try {
			byte [] outputLanguageRecord = mobileDictionaryStore.getRecord(rms_index_output_language_group);
			ByteArrayInputStream byteInStream = new ByteArrayInputStream(outputLanguageRecord); 
			DataInputStream dataInStream = new DataInputStream(byteInStream);
			for (int languageCounter = 0; 
			     languageCounter < DictionaryDataFile.numberOfAvailableLanguages; 
				 ++languageCounter) {
				boolean languageSetting = dataInStream.readBoolean();
				outputLanguage[languageCounter] = languageSetting;
			}
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
		catch (IOException e) {
			throw new DictionaryException(e);
		}
		return outputLanguage;
	}
	
	private byte [] getBooleanArrayRecord(boolean [] booleanArray) throws IOException {
		ByteArrayOutputStream byteOutStream = new ByteArrayOutputStream(); 
		DataOutputStream dataOutStream = new DataOutputStream(byteOutStream);
		for (int languageCounter = 0; 
	         languageCounter < booleanArray.length; 
		     ++languageCounter) {
			dataOutStream.writeBoolean(booleanArray[languageCounter]);
		}
		return byteOutStream.toByteArray();
	}
	
	public void setOutputLanguage(boolean [] outputLanguage) throws DictionaryException  {
		try {
			byte [] outputLanguageRecord = getBooleanArrayRecord(outputLanguage);
			mobileDictionaryStore.setRecord(rms_index_output_language_group, 
										    outputLanguageRecord,
					                        0,
					                        outputLanguageRecord.length);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
		catch (IOException e) {
			throw new DictionaryException(e);
		}
	}

	
	public boolean getStatistic() throws DictionaryException  {
		boolean  statistic;
		try {
			 byte [] statisticRecord = mobileDictionaryStore.getRecord(rms_index_statistic);
			 statistic = (statisticRecord[0] == 1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
		return statistic;
	}
	
	public void setStatistic(boolean statistic) throws DictionaryException  {
		try {
			byte statisticByte;
			
			if (statistic) 
				statisticByte = 1;
			else
				statisticByte = 0;
			mobileDictionaryStore.setRecord(rms_index_statistic, 
			                                new byte [] { (byte) statisticByte },
					                        0,
					                        1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
	}

	public boolean getBypassCharsetDecoding() throws DictionaryException  {
		boolean  bypassCharsetDecoding;
		try {
			 byte [] bypassCharsetDecodingRecord = mobileDictionaryStore.getRecord(rms_index_bypassCharsetDecoding);
			 bypassCharsetDecoding = (bypassCharsetDecodingRecord[0] == 1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
		return bypassCharsetDecoding;
	}
	
	public void setBypassCharsetDecoding(boolean bypassCharsetDecoding) throws DictionaryException  {
		try {
			byte bypassCharsetDecodingByte;
			
			if (bypassCharsetDecoding) 
				bypassCharsetDecodingByte = 1;
			else
				bypassCharsetDecodingByte = 0;
			mobileDictionaryStore.setRecord(rms_index_bypassCharsetDecoding, 
			                                new byte [] { (byte) bypassCharsetDecodingByte },
					                        0,
					                        1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
	}

	public boolean getIncrementalSearchDisabled() throws DictionaryException  {
		boolean  incrementalSearchDisabled;
		try {
			 byte [] incrementalSearchDisabledRecord = mobileDictionaryStore.getRecord(rms_index_incrementalSearchDisabled);
			 incrementalSearchDisabled = (incrementalSearchDisabledRecord[0] == 1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
		return incrementalSearchDisabled;
	}
	
	public void setIncrementalSearchDisabled(boolean incrementalSearchDisabled) throws DictionaryException  {
		try {
			byte incrementalSearchDisabledByte;
			
			if (incrementalSearchDisabled) 
				incrementalSearchDisabledByte = 1;
			else
				incrementalSearchDisabledByte = 0;
			mobileDictionaryStore.setRecord(rms_index_incrementalSearchDisabled, 
			                                new byte [] { (byte) incrementalSearchDisabledByte },
					                        0,
					                        1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
	}

	public int getMaxHits() throws DictionaryException  {
		int maxHits;
		try {
			 byte [] maxHitsRecord = mobileDictionaryStore.getRecord(rms_index_maxHits);
			 maxHits = maxHitsRecord[0];
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
		return maxHits;
	}
	
	public void setMaxHits(int maxHits) throws DictionaryException  {
		if (maxHits > 255) {
			throw new DictionaryException("Maximum hit number is 255");
		}
		try {
			mobileDictionaryStore.setRecord(rms_index_maxHits, 
			                                new byte [] { (byte) maxHits},
					                        0,
					                        1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
	}
	
	public int getFontSize() throws DictionaryException  {
		int fontSize;
		try {
			 byte [] fontSizeRecord = mobileDictionaryStore.getRecord(rms_index_fontSize);
			 fontSize = fontSizeRecord[0];
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
		return fontSize;
	}
	
	public void setFontSize(int fontSize) throws DictionaryException  {
		try {
			mobileDictionaryStore.setRecord(rms_index_fontSize, 
			                                new byte [] { (byte) fontSize},
					                        0,
					                        1);
		}
		catch (RecordStoreException e) {
			throw new DictionaryException(e);
		}
	}
	

}
