/*
DictionaryForMIDs - a free multi-language dictionary for mobile devices.
Copyright (C) 2005 Gert Nuber (Gert.Nuber@gmx.net)

GPL applies - see file COPYING for copyright statement.
*/

package de.kugihan.dictionaryformids.general;
import java.util.*;
import java.lang.*;
import java.io.*;


public abstract class Util {

	//tmp:
	public static void memCheck(String message) {
		if (getUtil().logLevel > logLevel1) {
			System.gc();
			getUtil().log(message + ": " + String.valueOf(Runtime.getRuntime().freeMemory()));
		}
	}
	
	private static Util utilObj; 
		
	public static Util getUtil() {
		return utilObj;
	}
	
	public static void setUtil(Util newUtilObj) {
		utilObj = newUtilObj;
	}
	
	protected abstract void outputMessage(String message); 
 
	public static final int    logLevel0 = 0;
	public static final int    logLevel1 = 1;
	public static final int    logLevel2 = 2;
	public static final int    logLevel3 = 3;
	public static final int    logLevelMin = logLevel0;
	public static final int    logLevelMax = logLevel3;
	protected int              logLevel = logLevelMin;

	public void log(String logMessage, int logLevelOutput) {
		if (logLevel >= logLevelOutput)
			outputMessage(logMessage + "\n");
	}
	
	public void log(String logMessage) {
		log(logMessage, logLevelMin);
	}
	
	public void logDebug(String logMessage) {
		log(logMessage, logLevel1);
	}
	
	public void log(Throwable thrown) {
		// log exception, unless it is a DictionaryInterruptedException (which can be raied during
		// normal conditions)
		if (! (thrown instanceof DictionaryInterruptedException)) {
			outputMessage("Thrown " + thrown.toString() + " / " + thrown.getMessage() + "\n");
			// if (logLevel >= logLevel1)
				thrown.printStackTrace();
		}
	}
	
	public void logTime(String message, long startTime) {
		long endTime = System.currentTimeMillis();
		long executionTime = endTime - startTime;
		log("Time for " + message + ": " + executionTime, logLevel2);
	}
	
	public void setLogLevel(int logLevelParam) {
		logLevel = logLevelParam;
	}

	public static boolean isSeparatorCharacter(char character) {
		return charIsLineWhitespace(character) ||
	    	   charIsPunctuation(character);
	}
	
	public static StringBuffer convertToLowerCase(StringBuffer string) {
		StringBuffer lowerCaseWord = new StringBuffer();
		for (int charPos = 0; charPos < string.length(); ++charPos) {
			lowerCaseWord.append(Character.toLowerCase(string.charAt(charPos)));
		}
		return lowerCaseWord;
	}
	
	public static void firstWord(StringBuffer words) {
		for (int charPos = 0; charPos < words.length(); ++charPos) {
			if (charIsLineWhitespace(words.charAt(charPos)) ||
			    charIsPunctuation(words.charAt(charPos))) {
				words.setLength(charPos);
				break;
			}
		}
	}
	
	public static boolean charIsLineWhitespace(char character) {
		boolean returnValue;
		if ( (character == '\t') ||
			 (character == ' ')) {
			returnValue = true;			
		}
		else {
			returnValue = false;
		}
		return returnValue;
	}

	// behaviour similar to J2SE Character.isWhitespace:
	public static boolean isWhitespace(char character) {
		boolean returnValue;
		if ( (character == '\t') ||
			 (character == '\n') || 
			 (character == ' ') || 
			 (character == '\u000C') || 
			 (character == '\u001C') || 
			 (character == '\u001D') || 
			 (character == '\u001E') || 
			 (character == '\u001F')) 
			 {
			returnValue = true;			
		}
		else {
			returnValue = false;
		}
		return returnValue;
	}

	public static String filterSuperflousWhitespaces(String expression) {
		String resultExpression = new String();
		boolean lastCharWasWhitespace = false;
		for (int charPos = 0; charPos < expression.length(); ++ charPos) {
			char character = expression.charAt(charPos);
			if (charIsLineWhitespace(character)) {
			    if (resultExpression.length() == 0) {
					// ignore whitspaces at the beginning of the expression
				}
				else if (charPos == expression.length() -1) {
					// ignore whitspaces at the end of the expression
				}
				else if (lastCharWasWhitespace)  {
					// ignore multiple whitespaces in sequence
				}
				else {
					resultExpression = resultExpression + character;  // todo: use StringBuffer instead of String
				}
				lastCharWasWhitespace = true;
			}
			else {
				resultExpression = resultExpression + character;
				lastCharWasWhitespace = false;
			}
		}
		return resultExpression;
	}
	
	static boolean charIsPunctuation(char character) {
		boolean returnValue;
		if ( (character == '!') ||
			 (character == '"') ||
			 (character == '§') ||
			 (character == '$') ||
			 (character == '%') ||
			 (character == '&') ||
			 (character == '/') ||
			 (character == '(') ||
			 (character == ')') ||
			 (character == '=') ||
			 (character == '?') ||
			 (character == ';') ||
			 (character == ':') ||
			 (character == ',') ||
			 (character == '.') ||
			 (character == '-') ||
			 (character == '+') ||
			 (character == '*') ||
			 (character == '#') ||
			 (character == '~') ||
			 (character == '@') ||
			 (character == '|') ||
			 (character == '<') ||
			 (character == '>') ||
			 (character == '\\')) {
			returnValue = true;			
		}
		else {
			returnValue = false;
		}
		return returnValue;
	}
	
	boolean stringEqualIgnoreCase(String string1, String string2) {
		boolean stringsAreEqual = false;
		if (string1.length() == string2.length()) {
			int stringLength = string1.length();
			int pos;
			for (pos = 0; pos < stringLength; ++pos) {
				if (Character.toUpperCase(string1.charAt(pos)) != Character.toUpperCase(string2.charAt(pos))) {
					break;
				}
			}
			if (pos == stringLength) {
				stringsAreEqual = true;
			}
		}
		return stringsAreEqual;
	}
	
	public void openProperties() throws DictionaryException  {
		// default is to do nothing
	}

	public void closeProperties() throws DictionaryException  {
		// default is to do nothing
	}

	public abstract String getDictionaryProperty(String propertyName);
	
	public String getDictionaryPropertyString(String propertyName, boolean optional) throws DictionaryException {
		String propertyValue = getDictionaryProperty(propertyName);
		if ((propertyValue == null) && (! optional)) {
			String errorText = "Property " + propertyName + " not found";
			throw new DictionaryException(errorText);
		}
		return propertyValue;
	}

	public String getDictionaryPropertyString(String propertyName) throws DictionaryException {
		return getDictionaryPropertyString(propertyName, false);
	}
	
	public int getDictionaryPropertyInt(String propertyName) throws DictionaryException {
		String propertyValue = getDictionaryPropertyString(propertyName);
		int propertyValueInt = Integer.valueOf(propertyValue).intValue();
		return propertyValueInt;
	}

	public char getDictionaryPropertyChar(String propertyName) throws DictionaryException {
		String propertyValue = getDictionaryPropertyString(propertyName);
		char propertyValueChar = getCharValueFromProperty(propertyName, propertyValue);
		return propertyValueChar;
	}

	public char getCharValueFromProperty(String propertyName, String propertyValue)  
	                   throws DictionaryException  {
		char propertyValueChar = '\t';
		if ((! propertyValue.startsWith("\'")) ||
			(! propertyValue.endsWith("\'"))) {
				String errorText = "Property "  + propertyName + " must start with \' and end with \' ";
				log(errorText);
				throw new DictionaryException(errorText);
		}
		String extractedValue = propertyValue.substring(1,propertyValue.length() - 1);
		boolean charPropertyNotWellFormed = false;
		if (extractedValue.startsWith("\\")) {
			if (extractedValue.length() != 2)
				charPropertyNotWellFormed = true;
			else if (extractedValue.endsWith("t"))
				propertyValueChar = '\t';
			else 
				charPropertyNotWellFormed = true;
		}
		else {
			if (extractedValue.length() != 1)
				charPropertyNotWellFormed = true;
			else
				propertyValueChar = extractedValue.charAt(0);
		}
		if (charPropertyNotWellFormed) {
			String errorText = "Property "  + propertyName + " must contain one character or \\t";
			log(errorText);
			throw new DictionaryException(errorText);
		}
		return propertyValueChar;
	}

	public String getDictionaryPropertyStringDefault(String propertyName, String defaultValue) 
				throws DictionaryException {
		String returnValueString;
		String propertyValue = getDictionaryPropertyString(propertyName, true);
		if (propertyValue != null) {
			returnValueString = propertyValue;
		}
		else {
			// property was not set
			returnValueString = defaultValue;
		}
		return returnValueString;
	}

	public char getDictionaryPropertyCharDefault(String propertyName, char defaultValue) 
			throws DictionaryException {
		char returnValueChar;
		String propertyValue = getDictionaryPropertyString(propertyName, true);
		if (propertyValue != null) {
			returnValueChar = getCharValueFromProperty(propertyName, propertyValue);
		}
		else {
			// property was not set
			returnValueChar = defaultValue;
		}
		return returnValueChar;
	}

	public int getDictionaryPropertyIntDefault(String propertyName, int defaultValue) 
    			throws DictionaryException {
		int returnValueInt;
		String propertyValue = getDictionaryPropertyString(propertyName, true);
		if (propertyValue != null) {
			returnValueInt = Integer.valueOf(propertyValue).intValue();
		}
		else {
			// property was not set
			returnValueInt = defaultValue;
		}
		return returnValueInt;
	}
		
	public boolean getDictionaryPropertyBooleanDefault(String propertyName, boolean defaultValue) 
	                         throws DictionaryException {
		boolean returnValueBoolean;
		String propertyValue = getDictionaryPropertyString(propertyName, true);
		if (propertyValue != null) {
			if (stringEqualIgnoreCase(propertyValue, "true")) {
				returnValueBoolean = true;
			}
			else if (stringEqualIgnoreCase(propertyValue, "false")) {
				returnValueBoolean = false;
			}
			else {
				String errorText = "Property "  + propertyName + " must be true or false";
				log(errorText);
				throw new DictionaryException(errorText);
			}
		}
		else {
			// property was not set
			returnValueBoolean = defaultValue;
		}
		return returnValueBoolean;
	}
	
	// For some devices the charset encoding that is configurated for DictionaryForMIDs
	// needs to be replaces with a device specific string. This is done with a call to
	// setDeviceCharEncoding. At initialisation determineCharEncoding needs to be called
	public static void determineCharEncoding() throws DictionaryException {
		// default is to do nothing
	}
	
	public static void setDeviceCharEncoding(String charEncoding) {
		// default is to do nothing
	}
}
