/*
DictionaryForMIDs - a free multi-language dictionary for mobile devices.
Copyright (C) 2005 Gert Nuber (Gert.Nuber@gmx.net)

GPL applies - see file COPYING for copyright statement.
*/

package de.kugihan.dictionaryformids.general;

import java.io.*;
import javax.microedition.lcdui.*;

import de.kugihan.dictionaryformids.dataaccess.DictionaryDataFile;
import de.kugihan.dictionaryformids.hmi_j2me.DictionaryForMIDs;


public class UtilMid extends Util {

	private static Form logForm;

	private Properties dictionaryForMIDsProperties;
	
	private static boolean oldStyleEncoding;
	public  static final String  oldStyleISO88591 = "ISO8859_1";
		
	public void openProperties() throws DictionaryException {
		try {
			dictionaryForMIDsProperties = new Properties();
			String propertyFileName = DictionaryDataFile.getPathDataFiles() + DictionaryDataFile.propertyFileName;
			InputStream propertyStream = getClass().getResourceAsStream(propertyFileName);
			if (propertyStream == null) {
				throw new DictionaryException("Property file could not be opened: " + DictionaryDataFile.propertyFileName);
			}
			else {
				dictionaryForMIDsProperties.load(propertyStream);
			}
		}
		catch (IOException e) {
			log("Error reading property file " + DictionaryDataFile.propertyFileName);
			throw new DictionaryException(e);
		}
	}

	public void closeProperties() throws DictionaryException {
		dictionaryForMIDsProperties = null;
	}

	public String getDictionaryProperty(String propertyName) {
		return dictionaryForMIDsProperties.getProperty(propertyName);
	}	
	
	public static void setLogForm(Form logFormParam) {
		logForm = logFormParam;
	}

	protected void outputMessage(String message) {
		StringItem outputMessageItem = new StringItem(null, message);
		if (DictionaryForMIDs.useMIDP20)
			outputMessageItem.setLayout(Item.LAYOUT_NEWLINE_BEFORE | Item.LAYOUT_2);
		logForm.append(outputMessageItem);
	}

	// old MIDP 1.0 phones sometimes use the string ISO8859_1 instead of ISO-8859-1
	public static void determineCharEncoding() throws DictionaryException {
		oldStyleEncoding = false;
		if (! DictionaryForMIDs.useMIDP20) {
			String supportedEncoding = System.getProperty("microedition.encoding");
			if (supportedEncoding == null) {
				throw new DictionaryException("System property microedition.encoding could not be read");
			}
			if (supportedEncoding.equals(oldStyleISO88591)) {
				oldStyleEncoding = true;
			}
		}
	}
	
	public static void setDeviceCharEncoding(String charEncoding) {
		if (charEncoding.equals("ISO-8859-1")) {
			if (oldStyleEncoding) {
				charEncoding = oldStyleISO88591;
			}
		}
	}
}
