/*
DictionaryForMIDs - a free multi-language dictionary for mobile devices.
Copyright (C) 2005 Gert Nuber (Gert.Nuber@gmx.net)

GPL applies - see file COPYING for copyright statement.
*/

package de.kugihan.dictionaryformids.hmi_j2me;

import javax.microedition.lcdui.*;
import javax.microedition.rms.*; 

import de.kugihan.dictionaryformids.dataaccess.DictionaryDataFile;
import de.kugihan.dictionaryformids.dataaccess.CsvFile;
import de.kugihan.dictionaryformids.general.DictionaryException;
import de.kugihan.dictionaryformids.general.SettingsStore;
import de.kugihan.dictionaryformids.general.Util;

public class DictionarySettingForm 
   extends Form 
   implements CommandListener,
              ItemStateListener {
	Command selectDictionaryCommand = new Command("Apply", Command.SCREEN, 4);

	Form callingForm;
	Display display;

	ChoiceGroup inputLanguageChoiceGroup;
	ChoiceGroup outputLanguageChoiceGroup;
	ChoiceGroup statisticChoiceGroup;
	ChoiceGroup fontSizeChoiceGroup;
	ChoiceGroup performanceChoiceGroup;

	protected String[] fontSizeStrings = new String [] { "Device default",
														  "Small",
														  "Medium",
														  "Large"};
	protected final int fontSizeIndexDefault = 0;
	protected final int fontSizeIndexSmall   = 1;
	protected final int fontSizeIndexMedium  = 2;
	protected final int fontSizeIndexLarge   = 3;	

	protected final int indexPerfCGIncrementalSearchDisabled = 0;
	protected final int indexPerfCGBypassCharsetDecoding = 1;

	DictionarySettingForm(Display displayParam,
			              Form callingFormParam) {
		super("Settings");

		callingForm = callingFormParam;
		display = displayParam;
		addCommand(selectDictionaryCommand);
		setCommandListener(this);
		
		String[] inputLanguageStrings = new String[DictionaryDataFile.numberOfInputLanguages];
		int inputLanguage = 0;
		for (int language = 0; language < DictionaryDataFile.numberOfAvailableLanguages; ++language) {
			if (DictionaryDataFile.supportedLanguages[language].isSearchable) {
				inputLanguageStrings[inputLanguage] = DictionaryDataFile.supportedLanguages[language].languageDisplayText;
				++inputLanguage;
			}
		}
		
		try {
			/* 
			 * input language
			 */
			inputLanguageChoiceGroup = new ChoiceGroup("From language",
													   Choice.EXCLUSIVE,
													   inputLanguageStrings,
													   null);
			DictionarySettings.setSelectedInputLanguage(SettingsStore.getSettingsStore().getInputLanguage());
			inputLanguageChoiceGroup.setSelectedIndex(DictionarySettings.getSelectedInputLanguage(), true);
			append(inputLanguageChoiceGroup);
						
			/* 
			 * output language
			 */
			String [] outputLanguageStrings = new String [DictionaryDataFile.numberOfAvailableLanguages];
			for (int languageIndex = 0; 
			     languageIndex < DictionaryDataFile.numberOfAvailableLanguages;
			     ++languageIndex) {
				outputLanguageStrings[languageIndex] = DictionaryDataFile.supportedLanguages[languageIndex].languageDisplayText;
			}
			outputLanguageChoiceGroup = new ChoiceGroup("To language",
													    Choice.EXCLUSIVE, // could also be Choice.MULTIPLE for multiple language output
														outputLanguageStrings,
														null);
			DictionarySettings.setSelectedOutputLanguage(SettingsStore.getSettingsStore().getOutputLanguage());
			outputLanguageChoiceGroup.setSelectedFlags(DictionarySettings.getSelectedOutputLanguage());
			append(outputLanguageChoiceGroup);

			/* 
			 * Font size
			 */
			if (DictionaryForMIDs.useMIDP20) {
				fontSizeChoiceGroup = new ChoiceGroup("Font size",
						                              Choice.POPUP,
						                              fontSizeStrings,
						                              null);
				DictionarySettings.setSelectedFontSize(SettingsStore.getSettingsStore().getFontSize());
				fontSizeChoiceGroup.setSelectedIndex(DictionarySettings.getSelectedFontSize(), true);
				append(fontSizeChoiceGroup);
			}
			
			/* 
			 * statistics
			 */
			String [] statisticStrings = new String[] { "Show statistics" };
			statisticChoiceGroup = new ChoiceGroup("Statistics",
					                                Choice.MULTIPLE,
					                                statisticStrings,
					                                null);
			DictionarySettings.setSelectedStatistic(SettingsStore.getSettingsStore().getStatistic());
			statisticChoiceGroup.setSelectedIndex(0, DictionarySettings.isSelectedStatistic());
			append(statisticChoiceGroup);
			
			/* 
			 * performance
			 */
			String [] performanceStrings = new String[] { "Disable incremental search",
					                                       "Bypass charset decoding" };
			performanceChoiceGroup = new ChoiceGroup("Performance",
					                                Choice.MULTIPLE,
					                                performanceStrings,
					                                null);
			DictionarySettings.setIncrementalSearchDisabled(SettingsStore.getSettingsStore().getIncrementalSearchDisabled());
			CsvFile.selectedBypassCharsetDecoding = SettingsStore.getSettingsStore().getBypassCharsetDecoding();
			performanceChoiceGroup.setSelectedIndex(indexPerfCGIncrementalSearchDisabled, DictionarySettings.isIncrementalSearchDisabled());
			performanceChoiceGroup.setSelectedIndex(indexPerfCGBypassCharsetDecoding, CsvFile.selectedBypassCharsetDecoding);
			append(performanceChoiceGroup);
			
			setItemStateListener(this);
		}
		catch (DictionaryException e) {
			Util.getUtil().log("Throwable " + e.getMessage() + "\ntostring " + e.toString());
		}
	}

	public void commandAction (Command c, Displayable s)  {
		try {
			// save settings in SettingStore
			// input language:
			DictionarySettings.setSelectedInputLanguage(inputLanguageChoiceGroup.getSelectedIndex());
			SettingsStore.getSettingsStore().setInputLanguage(DictionarySettings.getSelectedInputLanguage());		
			// output language:
			DictionarySettings.setSelectedOutputLanguage(new boolean [DictionaryDataFile.numberOfAvailableLanguages]);
			outputLanguageChoiceGroup.getSelectedFlags(DictionarySettings.getSelectedOutputLanguage());
			SettingsStore.getSettingsStore().setOutputLanguage(DictionarySettings.getSelectedOutputLanguage());		
			// font size:
			if (DictionaryForMIDs.useMIDP20) {
				int newSelectedFontSize = fontSizeChoiceGroup.getSelectedIndex();
				if (newSelectedFontSize != DictionarySettings.getSelectedFontSize()) {
					DictionarySettings.setSelectedFontSize(newSelectedFontSize);
					SettingsStore.getSettingsStore().setFontSize(DictionarySettings.getSelectedFontSize());
					// update font size on display
					MainForm.applicationMainForm.updateFonts();
				}
			}
			// statistic:
			boolean [] selectedStatisticFlags = new boolean[statisticChoiceGroup.size()];
			DictionarySettings.setSelectedStatistic((statisticChoiceGroup.getSelectedFlags(selectedStatisticFlags) > 0)); 
			SettingsStore.getSettingsStore().setStatistic(DictionarySettings.isSelectedStatistic());
			// update statistics
			MainForm.applicationMainForm.displayStatisticItems();
			// Performance settings
			// IncrementalSearchDisabled:
			boolean [] perfCGFlags = new boolean[performanceChoiceGroup.size()];
			performanceChoiceGroup.getSelectedFlags(perfCGFlags); 
			boolean oldIncrementalSearchDisabled = DictionarySettings.isIncrementalSearchDisabled(); 
			DictionarySettings.setIncrementalSearchDisabled(perfCGFlags[indexPerfCGIncrementalSearchDisabled]);
			SettingsStore.getSettingsStore().setIncrementalSearchDisabled(DictionarySettings.isIncrementalSearchDisabled());
			if (oldIncrementalSearchDisabled != DictionarySettings.isIncrementalSearchDisabled())
				MainForm.applicationMainForm.updateIncrementalSearchSetting();
			// BypassCharsetDecoding:
			CsvFile.selectedBypassCharsetDecoding = perfCGFlags[indexPerfCGBypassCharsetDecoding];
			SettingsStore.getSettingsStore().setBypassCharsetDecoding(CsvFile.selectedBypassCharsetDecoding);
			
			// return to calling form
			display.setCurrent(callingForm);
		}
		catch (DictionaryException e) {
			Util.getUtil().log("Throwable " + e.getMessage() + "\ntostring " + e.toString());
		}
	}
	
	public void itemStateChanged(Item item) {
		if (item == inputLanguageChoiceGroup) {
			int newSelectedInputLanguage = inputLanguageChoiceGroup.getSelectedIndex(); 
			boolean [] currentSelectedOutputLanguage = new boolean [DictionaryDataFile.numberOfAvailableLanguages];
			outputLanguageChoiceGroup.getSelectedFlags(currentSelectedOutputLanguage);
			if (currentSelectedOutputLanguage[newSelectedInputLanguage]) {
				currentSelectedOutputLanguage[newSelectedInputLanguage] = false;
				int newSelectedOutputLanguage = newSelectedInputLanguage + 1;
				if (newSelectedOutputLanguage == DictionaryDataFile.numberOfAvailableLanguages)
					newSelectedOutputLanguage = 0;
				currentSelectedOutputLanguage[newSelectedOutputLanguage] = true;
				outputLanguageChoiceGroup.setSelectedFlags(currentSelectedOutputLanguage);
			}
		}
	}
	
	public int getSelectedFontSize()
					throws DictionaryException {
		int fontSize;
		if (DictionarySettings.getSelectedFontSize() == fontSizeIndexDefault)
			fontSize = Font.getDefaultFont().getSize();
		else if (DictionarySettings.getSelectedFontSize() == fontSizeIndexLarge)
			fontSize = Font.SIZE_LARGE;
		else if (DictionarySettings.getSelectedFontSize() == fontSizeIndexMedium)
			fontSize = Font.SIZE_MEDIUM;
		else if (DictionarySettings.getSelectedFontSize() == fontSizeIndexSmall)
			fontSize = Font.SIZE_SMALL;
		else
			throw new DictionaryException("Invalid font size index");
		return fontSize;
	}
}
	
