package de.kugihan.dictionaryformids.hmi_j2me;

import java.util.Enumeration;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet; 

import de.kugihan.dictionaryformids.general.DictionaryException;
import de.kugihan.dictionaryformids.general.Util;
import de.kugihan.dictionaryformids.translation.TranslationExecution;
import de.kugihan.dictionaryformids.translation.TranslationResult;
import de.kugihan.dictionaryformids.translation.SingleTranslation;
import de.kugihan.dictionaryformids.translation.Translation;
import de.kugihan.dictionaryformids.translation.TranslationExecutionCallback;
import de.kugihan.dictionaryformids.dataaccess.DictionaryDataFile;

public class MainForm 
            extends    Form
			implements CommandListener,
	                   ItemStateListener,
	           TranslationExecutionCallback {
	
	public static MainForm applicationMainForm;

	private int  indexOfFirstTranslationItem; 
	private int  indexOfLastTranslationItem;
	private Font translationFromFont = null;
	private Font translationToFont = null;
	
	StringItem translationBreakReasonItem;
	StringItem responseTimeItem;
	StringItem freeMemoryItem;
	StringItem statisticItem;
	TextField  toBeTranslatedWordTextField;

	Command exitCommand = new Command("Exit", Command.EXIT, 3);
	Command selectDictionaryCommand = new Command("Settings", Command.SCREEN, 4);
	Command selectLanguageCommand = new Command("Language", Command.SCREEN, 5);
	Command helpCommand = new Command("Help", Command.SCREEN, 8);
	Command infoCommand = new Command("Info", Command.SCREEN, 9);
	Command translateCommand = new Command("Translate", Command.SCREEN, 1);
	Command newWordCommand = new Command("New word", Command.SCREEN, 2);
	
	Display display;
	DictionaryForMIDs  DictionaryForMIDsMidlet;
	
	DictionarySettingForm dictionarySettingFormObj;
	
	public MainForm(DictionaryForMIDs DictionaryForMIDsMidletParam) {
		super(DictionaryForMIDs.applicationName);
		applicationMainForm = this; 
		DictionaryForMIDsMidlet = DictionaryForMIDsMidletParam;
		display = Display.getDisplay(DictionaryForMIDsMidlet);
	}

	public void initialiseForm() 
	             throws DictionaryException {
		toBeTranslatedWordTextField = new TextField("Word for translation:", null, 60, TextField.ANY);
		append(toBeTranslatedWordTextField);

		translationBreakReasonItem = new StringItem(null, null);
		if (DictionaryForMIDs.useMIDP20)
			translationBreakReasonItem.setLayout(Item.LAYOUT_NEWLINE_BEFORE | Item.LAYOUT_2);
		indexOfFirstTranslationItem = append(translationBreakReasonItem);
		
		responseTimeItem = new StringItem(null, null);
		if (DictionaryForMIDs.useMIDP20)
			responseTimeItem.setLayout(Item.LAYOUT_NEWLINE_BEFORE | Item.LAYOUT_2);
		append(responseTimeItem);

		statisticItem = new StringItem(null, null);
		if (DictionaryForMIDs.useMIDP20)
			statisticItem.setLayout(Item.LAYOUT_NEWLINE_BEFORE | Item.LAYOUT_2);
		append(statisticItem);
		
		freeMemoryItem = new StringItem(null, null);
		if (DictionaryForMIDs.useMIDP20)
			freeMemoryItem.setLayout(Item.LAYOUT_NEWLINE_BEFORE | Item.LAYOUT_2);
		append(freeMemoryItem);

		addCommand(translateCommand);
		addCommand(newWordCommand);
		addCommand(exitCommand);
		addCommand(selectDictionaryCommand);
		// future plan: addCommand(selectLanguageCommand);
		addCommand(helpCommand);
		addCommand(infoCommand);

		Util.getUtil().log("Display elements created", Util.logLevelMax);

		dictionarySettingFormObj = new DictionarySettingForm(display, this);
		Util.getUtil().log("Settingsform created", Util.logLevelMax);

		updateFonts();

		displayStatisticItems();
		
		if (DictionaryForMIDs.useMIDP20)
			display.setCurrentItem(toBeTranslatedWordTextField); 

		checkForSonyEricssonWorkaround();
	}
	
	public void startForm() {
		setCommandListener(this);
		updateIncrementalSearchSetting();
		display.setCurrent(this);
		TranslationExecution.setTranslationExecutionCallback(this);
	}
	
	public void commandAction(Command c, Displayable s) {
		try {
			if (c == exitCommand)
				DictionaryForMIDsMidlet.notifyDestroyed();
			else if (c == translateCommand)
				TranslationExecution.executeTranslation(toBeTranslatedWordTextField.getString(), false);
			else if (c == selectDictionaryCommand)
				dictionarySetting();
			else if (c == newWordCommand)
				newWord();
			else if (c == selectLanguageCommand)
				selectLanguage();
			else if (c == helpCommand)
				showHelp();
			else if (c == infoCommand)
				showInfo();
		}
		catch (Throwable t) {
			Util.getUtil().log(t);
		}
	}

	public void itemStateChanged(Item item) {
		if (item == toBeTranslatedWordTextField) {
			// start background translation ("incremental translation")
			try
			{
				TranslationExecution.executeTranslation(toBeTranslatedWordTextField.getString(), true);
			}
			catch (Throwable t)
			{
				Util.getUtil().log(t);
			}
		}
	}
	
	public void newWord() {
		// empties the to be translated word and sets the focus on that field 
		toBeTranslatedWordTextField.setString("");
		if (DictionaryForMIDs.useMIDP20)
			display.setCurrentItem(toBeTranslatedWordTextField); 	
	}

	public void dictionarySetting() {
		Display.getDisplay(DictionaryForMIDsMidlet).setCurrent(dictionarySettingFormObj);	
	}
	
	public void selectLanguage() {
		String infoText = new String("Sorry:\n" +
				                     "only English supported for the moment");
		Alert infoPage = new Alert("Info", infoText, null, AlertType.INFO);
		infoPage.setTimeout(Alert.FOREVER);
		display.setCurrent(infoPage, this);
	}

	public void showHelp() {
		String helpText = new String("You can use wildchard characters for your search:\n" +
				                     Translation.wildcardAnySeriesOfCharacter + " matches any series of character\n" +
				                     Translation.wildcardAnySingleCharacter + " matches any single character\n\n" +
				                     "To switch off sub-expression search:\n" +
				                     Translation.noSearchSubExpression + " as first character inhibits sub-expression search at begin of expression\n" +
        							 Translation.noSearchSubExpression + " as last character inhibits sub-expression search at end of expression");
		Alert helpPage = new Alert("Help", helpText, null, AlertType.INFO);
		helpPage.setTimeout(Alert.FOREVER);
		Display.getDisplay(DictionaryForMIDsMidlet).setCurrent(helpPage, this);	
	}
	
	public void showInfo() {
		String versionStatusString = DictionaryForMIDsMidlet.versionStatus;
		if (versionStatusString == null)
			versionStatusString = new String("");
		else
			versionStatusString = "/ " + versionStatusString;
		String infoText = new String(DictionaryDataFile.infoText + "\n\n"+
				                     "Java application " + DictionaryForMIDsMidlet.applicationName + " copyright (C) by " + 
				                     "Gert Nuber:\nGert.Nuber@gmx.net\nhttp://dictionarymid.sourceforge.net"+ "\n\n" +
									 "Version " + DictionaryForMIDsMidlet.versionNumber + versionStatusString + "\n\n" +
									 "GNU General Public License applies for " + DictionaryForMIDsMidlet.applicationName + " Java application. See file COPYING.\n");
		// "Not allowed to change dictionary or Java application.")
		Alert infoPage = new Alert("Info", infoText, null, AlertType.INFO);
		infoPage.setTimeout(Alert.FOREVER);
		Display.getDisplay(DictionaryForMIDsMidlet).setCurrent(infoPage, this);
	}

	public void displayStatisticItems() {
		if (DictionarySettings.isSelectedStatistic()) {
			responseTimeItem.setLabel("Response time [ms]:");
			statisticItem.setLabel("Hits: ");
			freeMemoryItem.setLabel("Free Memory: ");
			freeMemoryItem.setText(String.valueOf(Runtime.getRuntime().freeMemory()));
		} else {
			statisticItem.setLabel(null);
			statisticItem.setText(null);
			responseTimeItem.setLabel(null);
			responseTimeItem.setText(null);
			freeMemoryItem.setLabel(null);
			freeMemoryItem.setText(null);			
		}	
	}

	// update the fonts of the translated texts:
	public void updateFonts() 
				throws DictionaryException {
		if (DictionaryForMIDs.useMIDP20) { // only supported starting with MIDP 2.0
			Font newTranslationFromFont = Font.getFont(Font.getDefaultFont().getFace(), 
					                                   Font.STYLE_UNDERLINED,
					                                   dictionarySettingFormObj.getSelectedFontSize());
			Font newTranslationToFont = Font.getFont(Font.getDefaultFont().getFace(), 
	                                                 Font.STYLE_BOLD,
					                                 dictionarySettingFormObj.getSelectedFontSize());
			if ((translationFromFont == null) && (translationToFont == null)) {
				// first time initialisation
				translationFromFont = newTranslationFromFont;
				translationToFont = newTranslationToFont;
			}
			else {
				for (int currentIndexTranslationItem = indexOfFirstTranslationItem;
					 currentIndexTranslationItem <= indexOfLastTranslationItem;
				     ++currentIndexTranslationItem) {
					Item item = get(currentIndexTranslationItem);
					if (! (item instanceof StringItem))
						throw new DictionaryException("StringItem expected");
					StringItem stringItem = (StringItem) item;
					Font lastFont = stringItem.getFont();
					if (lastFont == translationFromFont) {
						stringItem.setFont(newTranslationFromFont);
					}
					else if (lastFont == translationToFont) {
						stringItem.setFont(newTranslationToFont);
					}
					else {
						throw new DictionaryException("Unexpected Font found");
					}			
				}
				translationFromFont = newTranslationFromFont;
				translationToFont = newTranslationToFont;
			}
		}
	}
	
	public void updateIncrementalSearchSetting() {
		if (DictionarySettings.isIncrementalSearchDisabled()) {
			setItemStateListener(null);	
		}
		else {
			setItemStateListener(this);	
		}
	}
	
	public StringBuffer limitedString(StringBuffer inString) {
		// It may be that some devices have limitations on the StringItem string length.
		// This method may be used to limit the string lentgh. Currently no
		// limitation is done (till now no problem due to long strings was reported)
		return inString;
	}
	
	/*
	 * Display of translation results
	 */
	
	public void deletePreviousTranslationResult() {
		// delete result of previous translation
		for (int currentIndexTranslationItem = indexOfFirstTranslationItem;
			 currentIndexTranslationItem <= indexOfLastTranslationItem;
			 ++currentIndexTranslationItem) {
			delete(indexOfFirstTranslationItem);
		}
		indexOfLastTranslationItem = 0;
	}
		
	public void newTranslationResult(TranslationResult resultOfTranslation) {
		try {
			// display result of new translation
			int indexOfTranslationItem = indexOfFirstTranslationItem;
			if (resultOfTranslation.translationFound) {
				Enumeration translationsEnum = resultOfTranslation.translations.elements();
				while (translationsEnum.hasMoreElements()) {
					SingleTranslation singleTranslation = (SingleTranslation) translationsEnum.nextElement();
					String lineBreak = new String();
					if (! DictionaryForMIDs.useMIDP20)
						lineBreak = "\n";
					StringItem translationFromItem = new StringItem(null, 
							                                        limitedString(singleTranslation.fromText).toString() + lineBreak);
					if (DictionaryForMIDs.useMIDP20) {
						translationFromItem.setFont(translationFromFont);
						translationFromItem.setLayout(Item.LAYOUT_NEWLINE_BEFORE  | Item.LAYOUT_2);
					}
					insert(indexOfTranslationItem, translationFromItem);
					++indexOfTranslationItem;
					StringItem translationToItem = new StringItem(null, 
							                                      limitedString(singleTranslation.toText).toString() + lineBreak);
					if (DictionaryForMIDs.useMIDP20) {
						translationToItem.setFont(translationToFont);
						translationToItem.setLayout(Item.LAYOUT_NEWLINE_BEFORE | Item.LAYOUT_2);
					}
					insert(indexOfTranslationItem, translationToItem);
					++indexOfTranslationItem;
				}
			}
			else {
				StringItem translationNotFoundItem = new StringItem(null, "not found");
				if (DictionaryForMIDs.useMIDP20) {
					translationNotFoundItem.setFont(translationToFont);
					translationNotFoundItem.setLayout(Item.LAYOUT_NEWLINE_BEFORE);
				}
				insert(indexOfTranslationItem, translationNotFoundItem);
				++indexOfTranslationItem;
			}
			indexOfLastTranslationItem = indexOfTranslationItem -1; 
			
			if (resultOfTranslation.translationBreakOccurred && resultOfTranslation.displayBreakReason)
				translationBreakReasonItem.setText(resultOfTranslation.translationBreakReason);
			else
				translationBreakReasonItem.setText(null);
			
			Util.memCheck("results displayed: ");
			if (DictionarySettings.isSelectedStatistic()) {
				String hitsText = String.valueOf(resultOfTranslation.numberOfHits);
				statisticItem.setText(hitsText);
				responseTimeItem.setText(String.valueOf(resultOfTranslation.executionTime));
				System.gc();
				freeMemoryItem.setText(String.valueOf(Runtime.getRuntime().freeMemory()));
			}
			
			applySonyEricssonWorkaround();
		}
		catch (Throwable t) {
			Util.getUtil().log(t);
		}
	}

	
	// Hope someone will solve this SonyEricsson problem, so that the following workaround will
	// not be necessary any more !
	//
	// Workaround for SonyEricsson OutOfMemoryError that occurs when deleting old translation
	// items at the beginning of executeTranslation.
	// The workaround is to display an info-dialog after the translation result has been displayed
	// It seems that then no OutOfMemoryError is thrown.
	boolean sonyEricssonWorkaroundRequired = false;
	void checkForSonyEricssonWorkaround() {
		String platform = System.getProperty("microedition.platform");
		if (platform != null) {
			if (platform.startsWith("SonyEricsson")) {
				if (!platform.startsWith("SonyEricssonP")) { // P-Series does not have the OutOfMemoryError
					sonyEricssonWorkaroundRequired = true;
				}
			}
		}
	}
	void applySonyEricssonWorkaround() {
		if (sonyEricssonWorkaroundRequired) {
			showSonyEricssonWorkaroundInfo();
		}
	}
	public void showSonyEricssonWorkaroundInfo() {
		String workaroundText = new String("SE workaround");
		Alert workaroundPage = new Alert("workaround", workaroundText, null, AlertType.INFO);
		workaroundPage.setTimeout(10);  // display with duration of 10 ms
		Display.getDisplay(DictionaryForMIDsMidlet).setCurrent(workaroundPage, this);
	}

}
