/*
DictionaryForMIDs - a free multi-language dictionary for mobile devices.
Copyright (C) 2005 Gert Nuber (Gert.Nuber@gmx.net)

GPL applies - see file COPYING for copyright statement.
*/

package de.kugihan.dictionaryformids.translation;
import de.kugihan.dictionaryformids.dataaccess.*;
import de.kugihan.dictionaryformids.general.DictionaryException;
import de.kugihan.dictionaryformids.general.Util;
import de.kugihan.dictionaryformids.hmi_j2me.*;

public class Translation {

	boolean searchSubExpressionStart = true;
	boolean searchSubExpressionEnd = true;
	
	public Translation(boolean searchSubExpressionStartParam, 
					   boolean searchSubExpressionEndParam) {
		searchSubExpressionStart = searchSubExpressionStartParam;
		searchSubExpressionEnd = searchSubExpressionEndParam;
	}
	
	public static final char wildcardAnySeriesOfCharacter = '*';
	public static final char wildcardAnySingleCharacter = '?';
	public static final char noSearchSubExpression = '/';
	
	// returns true if character is a wildcard character
	static boolean isWildcardCharacter(char character) {
		return (character == wildcardAnySeriesOfCharacter) || (character == wildcardAnySingleCharacter);
	}
	
	// returns -1 if no wildcard character exists in word:
	static int positionFirstWildcardCharacter(String word) {
		int index;
		int indexWildcardAnySeriesOfCharacters = word.indexOf(wildcardAnySeriesOfCharacter);
		int indexWildcardAnySingleCharacter = word.indexOf(wildcardAnySingleCharacter);
		if ((indexWildcardAnySingleCharacter == -1) &&
			(indexWildcardAnySeriesOfCharacters == -1)) {
			index = -1;
		}
		else if ((indexWildcardAnySingleCharacter == -1) &&
			     (indexWildcardAnySeriesOfCharacters > -1)) {
				index = indexWildcardAnySeriesOfCharacters;
		}
		else if ((indexWildcardAnySingleCharacter > -1) &&
			     (indexWildcardAnySeriesOfCharacters == -1)) {
				index = indexWildcardAnySingleCharacter;
		}
		else if (indexWildcardAnySingleCharacter < indexWildcardAnySeriesOfCharacters) {
			index = indexWildcardAnySingleCharacter;
		}
		else {
			index = indexWildcardAnySeriesOfCharacters;			
		}
		return index;
	}
	static String expressionTillWildcard(String word) {
		return word.substring(0, positionFirstWildcardCharacter(word));
	}

	TranslationResult resultOfTranslation = new TranslationResult();
	
	long startTime;
	
	boolean translationIsCancelled = false;
	
	public TranslationResult getTranslationResult(String toBeTranslatedWord) {

		resultOfTranslation.numberOfHits = 0;
		startTime = System.currentTimeMillis();
		Util.memCheck("start translation: ");
		Normation normationObj = DictionaryDataFile.supportedLanguages[DictionarySettings.getSelectedInputLanguage()].normationObj;
		String toBeTranslatedWordNormated = normationObj.normateWord(new StringBuffer(toBeTranslatedWord)).toString();
		toBeTranslatedWordNormated = Util.filterSuperflousWhitespaces(toBeTranslatedWordNormated);
		
		if (toBeTranslatedWordNormated.length() > 0) {
			try { 
				String initialSearchExpression;
				boolean containsWildcard = (positionFirstWildcardCharacter(toBeTranslatedWordNormated) >= 0);
				if (containsWildcard)
					initialSearchExpression = expressionTillWildcard(toBeTranslatedWordNormated);
				else
					initialSearchExpression = toBeTranslatedWordNormated;
				int initialSearchExpressionLength = initialSearchExpression.length();
				
				/* 
				 * read searchlist file
				 */
				String languagePostfix = DictionaryDataFile.supportedLanguages[DictionarySettings.getSelectedInputLanguage()].languageFilePostfix;
				String searchListFileName = DictionaryDataFile.getPathDataFiles() + 
										    DictionaryDataFile.prefixSearchListFile + 
											languagePostfix +
											DictionaryDataFile.suffixSearchListFile; 
				CsvFile searchListFile = new ResourceCsvFile(searchListFileName,
											    DictionaryDataFile.searchListFileSeparationCharacter,
												DictionaryDataFile.searchListCharEncoding,
												DictionaryDataFile.searchListFileMaxSize);
				
				Util.memCheck("searchfile open: ");
				String indexFileNumber = null;
				boolean lastIndexFileSearched = false;
				searchListFile.setPositionBefore(initialSearchExpression);
				while (!searchListFile.endOfDictionaryReached) {
					String wordInIndex = searchListFile.getWord().toString();
					if (containsWildcard) {
						int endOfFirstPartWordInIndex = initialSearchExpressionLength;
						int wordInIndexLength = wordInIndex.length();
						if (wordInIndexLength < initialSearchExpressionLength)
							endOfFirstPartWordInIndex = wordInIndexLength; 
						String firstPartWordInIndex = wordInIndex.substring(0, endOfFirstPartWordInIndex);
						if (firstPartWordInIndex.startsWith(initialSearchExpression)) {
							if (! lastIndexFileSearched) {
								if (indexFileNumber != null) {									
									if (searchInIndexFileBreakCondition(toBeTranslatedWordNormated, indexFileNumber))
										break;
								}
							}
							indexFileNumber = searchListFile.getWord().toString();
							if (searchInIndexFileBreakCondition(toBeTranslatedWordNormated, indexFileNumber))
								break;
							lastIndexFileSearched = true;
						}
						else if (firstPartWordInIndex.compareTo(initialSearchExpression) > 0) {
							if (! lastIndexFileSearched)
								if (indexFileNumber != null) {
									if (searchInIndexFileBreakCondition(toBeTranslatedWordNormated, indexFileNumber))
										break;
								}
							break;
						}
						else {
							indexFileNumber = searchListFile.getWord().toString();
						}
					}
					else {
						if (wordInIndex.compareTo(initialSearchExpression) >= 0) {
							// the last index file was the right one
							if (indexFileNumber != null) {
								if (searchInIndexFileBreakCondition(toBeTranslatedWordNormated, indexFileNumber))
									break;
							}
							// the search is continued 
							// when wordInIndex and initialSearchExpression are equal strings 
							// or 
							// when wordInIndex starts with initialSearchExpression and is wordInIndex
							// is followed by a blank
							boolean continueSearch = false;
							if (wordInIndex.startsWith(initialSearchExpression)) {
								if (wordInIndex.length() > initialSearchExpression.length()) {
									if (wordInIndex.charAt(initialSearchExpression.length()) == ' ') 
										continueSearch = true;
								}
								else 
									continueSearch = true;  // strings are equal
							}
							if (continueSearch) {
								indexFileNumber = searchListFile.getWord().toString();
							}
							else {
								// search is finished
								break;
							}
						}
						else {
							indexFileNumber = searchListFile.getWord().toString();
						}
					}
				}

				if (searchListFile.endOfDictionaryReached) {
					// search in the last index file
					searchInIndexFileBreakCondition(toBeTranslatedWordNormated, indexFileNumber);
				}
				searchListFile = null; // to allow garbage collection
				long endTime = System.currentTimeMillis();
				resultOfTranslation.executionTime = endTime - startTime;
				Util.memCheck("end translation: ");
			}
			catch (Throwable t) {
				Util.getUtil().log(t);
			}
		}
		return resultOfTranslation;
	}
	
	public boolean searchInIndexFileBreakCondition(String toBeTranslatedWordNormated, String indexFileNumber) 
							throws DictionaryException  {
		if (translationBreakCondition()) {
			return true;
		}
		else {
			searchInIndexFile(toBeTranslatedWordNormated, indexFileNumber);
			return translationBreakCondition();
		}
	}
	
	public void searchInIndexFile(String toBeTranslatedWordNormated, String indexFileNumber) 
	                        throws DictionaryException  {
		Util.getUtil().logDebug("indexFileNumber " + indexFileNumber);
		String languagePostfix = DictionaryDataFile.supportedLanguages[DictionarySettings.getSelectedInputLanguage()].languageFilePostfix;
		String indexFileName = DictionaryDataFile.getPathDataFiles() + 
	                           DictionaryDataFile.prefixIndexFile +
							   languagePostfix +
							   indexFileNumber +
		                       DictionaryDataFile.suffixIndexFile;

		/* 
		 * read index file
		 */
		boolean containsWildcard = (positionFirstWildcardCharacter(toBeTranslatedWordNormated) >= 0);
		String searchExpression;
		String initialSearchExpression;
		if (containsWildcard) {
			searchExpression = removeDuplicateAnySeriesWildcard(toBeTranslatedWordNormated);
			initialSearchExpression = expressionTillWildcard(toBeTranslatedWordNormated);
		}
		else {
			searchExpression = toBeTranslatedWordNormated;
			initialSearchExpression = toBeTranslatedWordNormated;
		}
		int initialSearchExpressionLength = initialSearchExpression.length();
		String indexStringLine = null;
		Util.getUtil().logDebug("indexFileName " + indexFileName);
		
		CsvFile indexFile = new ResourceCsvFile(indexFileName,
						    DictionaryDataFile.indexFileSeparationCharacter,
							DictionaryDataFile.indexCharEncoding,
							DictionaryDataFile.indexFileMaxSize);

		Util.memCheck("indexfile open: ");
		indexFile.setPositionBefore(initialSearchExpression);
		while (! indexFile.endOfDictionaryReached) {
			String indexEntry = indexFile.getWord().toString();
			if (! containsWildcard) {
				if (indexEntry.compareTo(searchExpression) < 0) {
					// skip entry
					indexFile.skipRestOfLine();
				}
				// do an ordinary match on the entry
				else if (indexEntry.startsWith(searchExpression)) {
					if (noWildcardMatchRest(searchExpression, indexEntry)) {
						// entry found
						indexStringLine = indexFile.getWord().toString();
						getDictionaryEntry(indexStringLine);
					}
					else {
						indexFile.skipRestOfLine();
					}
				}
				else {
					break;
				}
			}
			else {
				int endOfFirstPartIndexEntry = initialSearchExpressionLength;
				int indexEntryLength = indexEntry.length();
				if (indexEntryLength < initialSearchExpressionLength)
					endOfFirstPartIndexEntry = indexEntryLength; 
				String firstPartIndexEntry = indexEntry.substring(0, endOfFirstPartIndexEntry);
				if (initialSearchExpression.compareTo(firstPartIndexEntry) < 0) {
					// no more matches possible:
					break;
				}
				// do a wildcard match on the entry
				if (wildcardMatch(indexEntry, 
						          0,
								  indexEntry.length(),
								  searchExpression, 
						          0, 
								  searchExpression.length())) {
					// Strings matched
					indexStringLine = indexFile.getWord().toString();
					getDictionaryEntry(indexStringLine);
				}
				else {
					indexFile.skipRestOfLine();
				}
			}
		}
	}
	
	// to improve response time, consecutive *-wildcards, such as in "un**ly" are removed
	// by one *-wildcard
	public String removeDuplicateAnySeriesWildcard(String wildcardExpression) {
		// todo: to be implemented
		return wildcardExpression;
	}

	// for perfomance reasons the string positions are passed in to the method wildcardMatch,
	// instead of creating new substrings for each recursion.
	// Note: this method is declared as final only for the reason of better performance
	public final boolean wildcardMatch(String toBeSearchedExpression,
								 int    positionCharacterToBeSearchedExpression,
								 int    lengthToBeSearchedExpression,
			                     String wildcardExpression,
			             		 int    positionCharacterWildcardExpression,
			             		 int    lengthWildcardExpression) {
		boolean expressionsMatch = false;

		if (positionCharacterWildcardExpression == lengthWildcardExpression) {
			if (positionCharacterToBeSearchedExpression == lengthToBeSearchedExpression) {
				// matched till to the last charcter
				expressionsMatch = true;
			} 
			else {
				// length is different: no match unless the next character in toBeSearchedExpression
				// is a blank character (and searchSubExpressionEnd is true)
				if ((lengthToBeSearchedExpression > positionCharacterToBeSearchedExpression) && searchSubExpressionEnd) {
					if (toBeSearchedExpression.charAt(positionCharacterToBeSearchedExpression) == ' ') {
						expressionsMatch = true;
					}
					else {
						expressionsMatch = false;
					}
				}
				else {
					expressionsMatch = false;
				}
			}
		}
		else if (positionCharacterToBeSearchedExpression == lengthToBeSearchedExpression) {
			// length is different: no match
			// but if the rest of the wilcard expression is wildcardAnySeriesOfCharacter, then
			// the expressions still match
			boolean restIsAnySeriesOfCharacter = true;
			for (int position = positionCharacterWildcardExpression;
			     position < lengthWildcardExpression;
				 ++position) {
				if (wildcardExpression.charAt(position) != wildcardAnySeriesOfCharacter) {
					restIsAnySeriesOfCharacter = false;
					break;
				}
			}
			expressionsMatch = restIsAnySeriesOfCharacter;
		}
		else {
			char characterToBeSearchedExpression = 
				              toBeSearchedExpression.charAt(positionCharacterToBeSearchedExpression);
			char characterWildcardExpression = 
				              wildcardExpression.charAt(positionCharacterWildcardExpression);

			if (characterWildcardExpression == wildcardAnySingleCharacter) {
				// matches to any character: check remaining part of the expressions
				expressionsMatch = wildcardMatch(toBeSearchedExpression,
						 						 positionCharacterToBeSearchedExpression + 1,
												 lengthToBeSearchedExpression,
												 wildcardExpression,
												 positionCharacterWildcardExpression + 1,
												 lengthWildcardExpression);
			}
			else if (characterWildcardExpression == wildcardAnySeriesOfCharacter) {
				// see if the rest of the wildcardExpression matches to any of the remaining 
				// characters of toBeSearchedExpression
				for (int position = positionCharacterToBeSearchedExpression;
				     position <= lengthToBeSearchedExpression;
					 ++position) {
					expressionsMatch = wildcardMatch(toBeSearchedExpression,
												     position,
													 lengthToBeSearchedExpression,
													 wildcardExpression,
													 positionCharacterWildcardExpression + 1,
													 lengthWildcardExpression);
					if (expressionsMatch)
						// match for expressions found, no need to continue search
						break;
				}
			}
			else if (characterWildcardExpression == characterToBeSearchedExpression) {
				// characters match: check remaining part of the expressions
				expressionsMatch = wildcardMatch(toBeSearchedExpression,
												 positionCharacterToBeSearchedExpression + 1,
												 lengthToBeSearchedExpression,
												 wildcardExpression,
												 positionCharacterWildcardExpression + 1,
												 lengthWildcardExpression);
			}
			
		}
		
		return expressionsMatch;
	}

	// checks if both strings are of equal length or if indexEntry is followed by a blank
	// character (and searchSubExpressionEnd is true)
	public boolean noWildcardMatchRest(String searchExpression, String indexEntry) {
		int searchExpressionLength = searchExpression.length();
		int indexEntryLength = indexEntry.length();
		if (searchExpressionLength == indexEntryLength) {
			return true;
		}
		if ((indexEntryLength > searchExpressionLength) && searchSubExpressionEnd) {
			if (indexEntry.charAt(searchExpressionLength) == ' ') {
				return true;
			}
		}
		// false in the other cases
		return false;
	}

	public void getDictionaryEntry(String indexStringLine) 
					throws DictionaryException {
		int posIndexFileSeparatorIndexEntries;
		int posFirstCharIndexString = 0;
		int posLastCharIndexString;
		do {
			posIndexFileSeparatorIndexEntries = 
				indexStringLine.indexOf(DictionaryDataFile.indexFileSeparatorIndexEntries, 
						                posFirstCharIndexString);
			if (posIndexFileSeparatorIndexEntries == -1)
				posLastCharIndexString = indexStringLine.length();
			else 
				posLastCharIndexString = posIndexFileSeparatorIndexEntries;
			String indexString = indexStringLine.substring(posFirstCharIndexString, 
					                                       posLastCharIndexString);
			posFirstCharIndexString = posLastCharIndexString + 1;
			int posIndexFileSeparatorFileNumberToPosition = 
			        indexString.indexOf(DictionaryDataFile.indexFileSeparatorFileNumberToPosition);
			String directoryFileNumber = indexString.substring(0, posIndexFileSeparatorFileNumberToPosition);
			int posIndexFileSeparatorFilePositionToSearchIndicator = 
				    indexString.indexOf(DictionaryDataFile.indexFileSeparatorFilePositionToSearchIndicator, 
				    		            posIndexFileSeparatorFileNumberToPosition + 1);
			if (posIndexFileSeparatorFilePositionToSearchIndicator == -1) {
				throw new DictionaryException("Indexfile has no searchindicator - use DictionaryGeneration 2.4.4 or newer");
			}
			int positionInDirectoryFile = 
				 Integer.valueOf(indexString.substring((posIndexFileSeparatorFileNumberToPosition + 1), 
						                               posIndexFileSeparatorFilePositionToSearchIndicator)).intValue();
			SearchIndicator searchIndicatorObj = new SearchIndicator(indexString.charAt(posIndexFileSeparatorFilePositionToSearchIndicator + 1));
			// if the index entry comes from a subexpression but searchSubExpressionStart is false then this entry is ignored
			if (!(searchSubExpressionStart || searchIndicatorObj.isBeginOfExpression())) {
				// just do nothing: don't get the corresponding dictionary entry
			}
			else {
				// ok, get the corresponding dictionary entry 
				String postfixDictionaryFile;
				LanguageDefinition supportedLanguage = DictionaryDataFile.supportedLanguages[DictionarySettings.getSelectedInputLanguage()]; 
				if (supportedLanguage.hasSeparateDictionaryFile) {
					// use the file postfix also for the dictionary file
					postfixDictionaryFile =
						supportedLanguage.languageFilePostfix;
				}
				else {
					// no suffix: empty string
					postfixDictionaryFile = "";
				}
				
				if (! translationBreakCondition()) {
					String dictionaryFileName = DictionaryDataFile.getPathDataFiles() + 
		                                        DictionaryDataFile.prefixDictionaryFile +
		                                        postfixDictionaryFile +
												directoryFileNumber +
		                                        DictionaryDataFile.suffixDictionaryFile;
					
					/*
					 *  read dictionary file
					 */ 
					Util.getUtil().logDebug("dictionaryFileName " + dictionaryFileName);
					Util.getUtil().logDebug("position " + String.valueOf(positionInDirectoryFile));
					getTranslation(dictionaryFileName, 
							       positionInDirectoryFile, 
							       searchIndicatorObj.isBeginOfExpression());
				}
				else {
					break;
				}
			}
		}
		while (posIndexFileSeparatorIndexEntries > 0);
	}
	
	public void getTranslation(String dictionaryFileName,
							   int positionInDirectoryFile,
							   boolean foundAtBeginOfExpression) 
	throws DictionaryException
	{
		CsvFile dictionaryFile = new ResourceCsvFile(dictionaryFileName, 
			    							         DictionaryDataFile.dictionaryFileSeparationCharacter,
													 DictionaryDataFile.dictionaryCharEncoding,
													 DictionaryDataFile.dictionaryFileMaxSize,
													 positionInDirectoryFile);

		Util.memCheck("dictionaryfile open: ");
		StringBuffer fromText = null; 
		StringBuffer toText = null; 
		
		for (int indexLanguage = 0;
  	         indexLanguage < DictionaryDataFile.numberOfAvailableLanguages;
	         ++indexLanguage) {
			StringBuffer word = dictionaryFile.getWord();
			
			if (DictionarySettings.getSelectedInputLanguage() == indexLanguage)
				fromText = word;
			if (DictionarySettings.getSelectedOutputLanguage(indexLanguage)) { // todo: handle more than one language
				toText = word;
			}
		}
		addTranslation(fromText, toText, foundAtBeginOfExpression);
		dictionaryFile = null; // to allow garbage collection
	}

	public void addTranslation(StringBuffer fromText, 
			                   StringBuffer toText,
			                   boolean foundAtBeginOfExpression) {
		if (toText.length() > 0) {
			convertFieldAndLineSeparatorChars(toText);
			/* Some dictionaries contain the same translation twice: such duplicate translations
			   are included one one time in the result. */
			boolean isDuplicate = false;
			/* Uncommented because this may be a performance issue if plenty of results are found 
			String fromTextString = fromText.toString(); 
			String toTextString = toText.toString(); 
			for (int indexTranslation = 0; 
			     indexTranslation < resultOfTranslation.translations.size(); 
				 ++indexTranslation) {
				SingleTranslation translation = (SingleTranslation) resultOfTranslation.translations.elementAt(indexTranslation);
				if (fromTextString.equals(translation.fromText.toString()) &&  
					toTextString.equals(translation.toText.toString())) {
					isDuplicate = true;
					break;
				} 
			} */
			if (! isDuplicate) {
				SingleTranslation singleTranslation = new SingleTranslation(fromText, toText);
				// if the translation was found at the beginning then put at the top of the result list 
				if (foundAtBeginOfExpression) {
					// put at the top
					resultOfTranslation.translations.insertElementAt(singleTranslation, 0);
				}
				else {
					// put at the end
					resultOfTranslation.translations.addElement(singleTranslation);
				}
				resultOfTranslation.translationFound = true;	
				++resultOfTranslation.numberOfHits;
			}
		}
	}

	public void convertFieldAndLineSeparatorChars(StringBuffer text) {
		int pos = 0;
		int textlength = text.length();
		while (pos < textlength) {
			char currentCharacter = text.charAt(pos);
			if (currentCharacter == '\\') {
				++pos;
				if (pos < textlength) {
					currentCharacter = text.charAt(pos);
					if (currentCharacter == '\\') {
						text.deleteCharAt(pos);
					}
					else if (currentCharacter == 'n') {
						text.setCharAt(pos-1, '\n');
						text.deleteCharAt(pos);
					}
					else if (currentCharacter == 't') {
						text.setCharAt(pos-1, '\t');
						text.deleteCharAt(pos);
					}
				}
				textlength = text.length(); // determine new length in case of deleteCharAt
			}
			else {
				++pos;
			}
		}
	}

	boolean translationBreakCondition() {
		if (resultOfTranslation.numberOfHits >= DictionarySettings.getMaxHits()) { 
			resultOfTranslation.translationBreakOccurred = true;
			resultOfTranslation.translationBreakReason = "max nr of hits reached";
			resultOfTranslation.displayBreakReason = true;
			return true;
		}
		else if (System.currentTimeMillis() - startTime >= DictionarySettings.getDurationForCancelSearch()) {
			resultOfTranslation.translationBreakOccurred = true;
			resultOfTranslation.translationBreakReason = "max execution time reached";
			resultOfTranslation.displayBreakReason = true;
			return true;
		}
		else if (translationIsCancelled) {
			resultOfTranslation.translationBreakOccurred = true;
			resultOfTranslation.translationBreakReason = "cancel received";
			resultOfTranslation.displayBreakReason = false;
			return true;
		}
		else {
			return false;
		}
	}
	
	// called from another thread: 
	public void cancelTranslation() {
		// set flag for cancelled translation
		// note: no "synchronized" is done for writing/reading translationIsCancelled: to my
		// understanding this is not required for writing/reading plain variables.
		translationIsCancelled = true;
	}
}
