package de.kugihan.dictionaryformids.translation;

import de.kugihan.dictionaryformids.general.*;
import de.kugihan.dictionaryformids.hmi_j2me.DictionarySettings;

public class TranslationExecution {

	protected static TranslationThread lastTranslationThread = null;
	
	protected static TranslationExecutionCallback translationResultHMIObj;
	
	public static void setTranslationExecutionCallback(TranslationExecutionCallback translationResultHMIObjParam) {
		translationResultHMIObj = translationResultHMIObjParam;
	}
	
	public synchronized static void executeTranslation(String toBeTranslatedWordTextParam, boolean executeInBackground)
	throws DictionaryException
	{
		// cancel last translation thread that may still be running
		if (lastTranslationThread != null) {
			lastTranslationThread.cancelTranslation();
			lastTranslationThread = null;
		}
		TranslationThread newTranslationThread = new TranslationThread(toBeTranslatedWordTextParam, executeInBackground);
	    if (executeInBackground) {
	    	// start new translation thread
	    	Thread executionThread = new Thread(newTranslationThread);
	    	newTranslationThread.setOwnExecutionThread(executionThread);
	    	lastTranslationThread = newTranslationThread;
	    	executionThread.start();
	    }
	    else {
	    	newTranslationThread.runInForeground();
	    }
	}
}

class TranslationThread implements Runnable {

	protected boolean translationIsCancelled = false;  // indicates whether the running translation should be aborted
	protected Translation translate = null;
	protected Thread ownExecutionThread;
	protected String toBeTranslatedWordText;
	protected boolean executeInBackground;
	
	public TranslationThread(String toBeTranslatedWordTextParam,
			                 boolean executeInBackgroundParam) {
		toBeTranslatedWordText = toBeTranslatedWordTextParam;
		executeInBackground = executeInBackgroundParam;
	}
	
	public void setOwnExecutionThread(Thread threadParam) {
		ownExecutionThread = threadParam;
	}
	
	// called from separate thread:
	public void run()
	{
		try
		{
			doTranslation();			
		}
		catch (Throwable t)
		{
			Util.getUtil().log(t);
		}
	}
	
	// called directly (not from separate thread):
	public void runInForeground()
	throws DictionaryException
	{
		doTranslation();
	}
	
	protected void doTranslation()
	throws DictionaryException
	{
		// delete previous translation result
		synchronized(this) {
			if (!translationIsCancelled)
				TranslationExecution.translationResultHMIObj.deletePreviousTranslationResult();
		}
		
		if (executeInBackground) {
		    // set a low priority for the thread
			ownExecutionThread.setPriority(Thread.NORM_PRIORITY - 2);
		}
		
		// get new translation
		if (toBeTranslatedWordText.length() > 0) { 
			// check if there is the noSearchSubExpression at the beginning or the end of the
			// to be translated word
			boolean searchSubExpressionStart = true;
			boolean searchSubExpressionEnd = true;
			if (toBeTranslatedWordText.charAt(0) == Translation.noSearchSubExpression) {
				searchSubExpressionStart = false;
				toBeTranslatedWordText = toBeTranslatedWordText.substring(1);
			}
			else {
				searchSubExpressionStart = true;
			}
			if (toBeTranslatedWordText.charAt(toBeTranslatedWordText.length() - 1) == Translation.noSearchSubExpression) {
				searchSubExpressionEnd = false;
				toBeTranslatedWordText = toBeTranslatedWordText.substring(0, toBeTranslatedWordText.length() - 1);
			}
			else {
				searchSubExpressionEnd = true;
			}
			translate = new Translation(searchSubExpressionStart, searchSubExpressionEnd);
			TranslationResult resultOfTranslation = null;
			
			if (!translationIsCancelled)
				resultOfTranslation = translate.getTranslationResult(toBeTranslatedWordText);
			
			// display new result
			synchronized(this) {
				if (!translationIsCancelled)
					TranslationExecution.translationResultHMIObj.newTranslationResult(resultOfTranslation);
			}
			translate = null;
		}
	}

	public synchronized void cancelTranslation() {
		// cancels a running translation
		
		// set flag for cancelled translation
		translationIsCancelled = true;
		
		// forward cancelling to translate-object
		// note: no "synchronized" is done for writing/reading the translate-reference: to my
		// understanding this is not required for writing/reading object references.
		if (translate != null)
			translate.cancelTranslation();
		
		// Send interrupt to translation thread
		if (DictionarySettings.isCldc11()) {
			ownExecutionThread.interrupt();
		}
	}
}